`timescale 1ns / 1ps
module SequenceBlock(
	output validOut,
	output [63:0] elementOut,
	output [9:0] memAddrOut,
	input validIn,
	input [4:0] memAddrIn,
	input [31:0] i,
	input [31:0] a1,
	input [31:0] d,
	input clock
);

reg validRegA, validRegB, validRegC;
reg [31:0]a1RegA, a1RegB;
reg [31:0]differenceReg, productReg, sumReg;
reg [31:0]dReg;
reg [4:0]memAddrRegA;
reg [4:0]memAddrRegB;
reg [4:0]memAddrRegC;

wire validAB, validBC;
wire [31:0]a1AB, a1IntoAdder;
wire [31:0]difference, multDifferenceIn, dIntoMult;
wire [63:0]multOut, productIntoAdder, sum;
wire [4:0]memAddrAB;
wire [4:0]memAddrBC;

assign validAB = validRegA;
assign validBC = validRegB;
assign validOut = validRegC;

assign memAddrAB = memAddrRegA;
assign memAddrBC = memAddrRegB;
assign memAddrOut = memAddrRegC;

assign a1AB = a1RegA;
assign a1IntoAdder = a1RegB;
assign difference = i - 1'b1;
assign multDifferenceIn = differenceReg;
assign dIntoMult = dReg;
assign multOut = multDifferenceIn * dIntoMult;
assign productIntoAdder = productReg;
assign sum = productIntoAdder + a1IntoAdder;
assign elementOut = sumReg;


// First pipeline stage:
always @(posedge clock) begin
	validRegA <= validIn;
	memAddrRegA <= memAddrIn;
	a1RegA <= a1;
	differenceReg <= difference;
	dReg <= d;
end

// Second pipeline stage:
always @(posedge clock) begin
	validRegB <= validAB;
	memAddrRegB <= memAddrAB;
	a1RegB <= a1AB;
	productReg <= multOut;
end

// Third pipeline stage:
always @(posedge clock) begin
	validRegC <= validBC;
	memAddrRegC <= memAddrBC;
	sumReg <= sum;
end

endmodule
